import ReactDOMServer from 'react-dom/server'
import { GraphQL } from '../universal/GraphQL'
export async function ssr(
  graphql,
  node,
  render = ReactDOMServer.renderToStaticMarkup
) {
  if (!(graphql instanceof GraphQL))
    throw new Error('ssr() argument 1 must be a GraphQL instance.')
  if (arguments.length < 2)
    throw new Error('ssr() argument 2 must be a React node.')
  if (typeof render !== 'function')
    throw new Error('ssr() argument 3 must be a function.')
  graphql.ssr = true

  async function recurse() {
    const string = render(node)
    const operations = Object.values(graphql.operations)

    if (operations.length) {
      await Promise.all(operations)
      return recurse()
    } else {
      delete graphql.ssr
      return string
    }
  }

  return recurse()
}
